"""Checker for using assertTrue/False instead of a more precise assert.

Hattip to Ned Batchelder for the idea:
http://nedbatchelder.com/blog/201505/writing_pylint_plugins.html
"""
import astroid

from pylint.interfaces import IAstroidChecker
from pylint.checkers import BaseChecker, utils

from .common import BASE_ID, check_visitors


def register_checkers(linter):
    """Register checkers."""
    linter.register_checker(AssertChecker(linter))


@check_visitors
class AssertChecker(BaseChecker):
    """
    Implements a few pylint checks on unitests asserts - making sure the right
    assert is used if assertTrue or assertFalse are misused.
    """

    __implements__ = (IAstroidChecker,)

    name = "assert-checker"

    AFFECTED_ASSERTS = ["assertTrue", "assertFalse"]

    BETTER_COMPARES = {
        "==": "assertEqual",
        "!=": "assertNotEqual",
        "in": "assertIn",
        "not in": "assertNotIn",
        "<": "assertLess",
        "<=": "assertLessEqual",
        ">": "assertGreater",
        ">=": "assertGreaterEqual",
        "is": "assertIs",
        "is not": "assertIsNot",
    }

    BETTER_NONE_COMPARES = {
        "==": "assertIsNone",
        "is": "assertIsNone",
        "!=": "assertIsNotNone",
        "is not": "assertIsNotNone",
    }

    INVERTED_PAIRS = [
        ("assertEqual", "assertNotEqual"),
        ("assertIn", "assertNotIn"),
        ("assertLess", "assertGreaterEqual"),
        ("assertGreater", "assertLessEqual"),
        ("assertIs", "assertIsNot"),
        ("assertIsNone", "assertIsNotNone"),
    ]

    INVERTED = {}
    for yup, nope in INVERTED_PAIRS:
        INVERTED[yup] = nope
        INVERTED[nope] = yup

    MESSAGE_ID = "wrong-assert-type"
    msgs = {("C%d90" % BASE_ID): ("%s", MESSAGE_ID, "Use assert(Not)Equal instead of assertTrue/False")}

    @utils.check_messages(MESSAGE_ID)
    def visit_call(self, node):
        """
        Check that various assertTrue/False functions are not misused.
        """
        if not isinstance(node.func, astroid.Attribute):
            # If it isn't a getattr ignore this. All the assertMethods are
            # attributes of self:
            return

        if node.func.attrname not in self.AFFECTED_ASSERTS:
            # Not an attribute / assert we care about
            return

        first_arg = node.args[0]
        existing_code = "%s(%s)" % (node.func.attrname, first_arg.as_string())

        if isinstance(first_arg, astroid.Compare):
            if len(first_arg.ops) > 1:
                # This is a chained comparison, which we can't do anything with.
                return

            compare = first_arg.ops[0][0]
            right = first_arg.ops[0][1]
            if isinstance(right, astroid.Const) and right.value is None:
                # Comparing to None, handle specially.
                better = self.BETTER_NONE_COMPARES[compare]
            else:
                better = self.BETTER_COMPARES[compare]

            if node.func.attrname == "assertFalse":
                better = self.INVERTED[better]
            self.add_message(self.MESSAGE_ID, args="%s should be %s" % (existing_code, better), node=node)



"""Test range_check.py"""

import pytest

from .pylint_test import run_pylint


@pytest.mark.parametrize("range_name", ["range", "xrange"])
def test_range(range_name):
    source = (
        """\
        START, STOP, STEP = 0, 10, 1
        # Bad
        range(0, 10)        #=A
        range(0, STOP)      #=B
        range(0, 10, 1)     #=C
        range(0, STOP, 1)   #=D
        range(10, 20, 1)    #=E

        # Good
        range(10)
        range(1, 10)
        range(0, 10, 2)
        range(1, 10, 2)

        # no message when variables are involved
        range(START, 100)
        range(START, STOP)
        range(0, 10, STEP)

        # if it has four arguments, we don't know what's going on...
        range(0, 10, 1, "something")

        # trickier cases
        range("something", "or other")
        [range][0](0, 10)
        some_other_function(0, 10)

    """
    ).replace("range", range_name)
    msg_ids = "simplifiable-range"
    messages = run_pylint(source, msg_ids)

    expected = {
        f"A:simplifiable-range:{range_name}() call could be single-argument",
        f"B:simplifiable-range:{range_name}() call could be single-argument",
        f"C:simplifiable-range:{range_name}() call could be single-argument",
        f"D:simplifiable-range:{range_name}() call could be single-argument",
        f"E:simplifiable-range:{range_name}() call could be two-argument",
    }
    assert expected == messages



"""The edx_lint update command: write any file that exists."""

from edx_lint.write import write_file
from edx_lint.metadata import existing_known_files

def update_main(argv_unused):  # pylint: disable=unused-argument
    """
    update
        Re-write any edx_lint-written files that exists on disk.
    """

    for filename in existing_known_files():
        ret = write_file(filename, output_fn=print)
        if ret != 0:
            return ret
    return 0



"""The edx_lint check command."""

import os
import re

from edx_lint.metadata import existing_known_files
from edx_lint.tamper_evident import TamperEvidentFile


def check_main(argv):
    """
    check [FILENAME ...]
        Check that FILENAME has not been edited since writing.
        If FILENAME is omitted, all existing files are checked.
    """
    if len(argv) >= 1:
        filenames = argv
    else:
        filenames = list(existing_known_files())

    ret = 0

    for filename in filenames:
        if os.path.exists(filename):
            tef = TamperEvidentFile(filename)
            if tef.validate():
                print(f"{filename} is good")
            else:
                with open(filename, encoding="utf-8") as f:
                    text = f.read()
                if re.search(r"generated by edx[-_]lint", text):
                    print(f"{filename} seems to have been edited")
                    ret = 1
                else:
                    print(f"{filename} wasn't written by edx_lint")
        else:
            print(f"{filename} doesn't exist")
            ret = 2

    return ret



"""Test i18n_check.py"""

from .pylint_test import run_pylint


def test_i18n_checker():
    source = """\
        _("This is fine")
        _("Hello"+"There")          #=A
        _(17)                       #=B
        _("Hi, {0}".format(name))   #=C
        gettext("hi, %s" % name)    #=D
        foobar(12)
    """

    msg_ids = "translation-of-non-string"
    messages = run_pylint(source, msg_ids)
    expected = {
        "A:translation-of-non-string:i18n function _() must be called with a literal string",
        "B:translation-of-non-string:i18n function _() must be called with a literal string",
        "C:translation-of-non-string:i18n function _() must be called with a literal string",
        "D:translation-of-non-string:i18n function gettext() must be called with a literal string",
    }
    assert expected == messages



""" Tests for the command line executable. """

import contextlib
import io
import os
import shutil
import tempfile
import unittest
from unittest.mock import patch

from edx_lint.cmd import main


PYLINTRC = "pylintrc"
PYLINTRC_TWEAKS = "pylintrc_tweaks"
PYLINTRC_BACKUP = "pylintrc_backup"

EDITORCONFIG = ".editorconfig"

JUST_TESTING = "just_for_testing.txt"


@contextlib.contextmanager
def capture_output():
    """
    Capture all of stdout and stderr, and return a StringIO.

    Note: using this means the output doesn't go to the real stdout. If you
    use the -s switch to see output, you won't see output inside this context.

    """
    output = io.StringIO()
    with contextlib.redirect_stdout(output):
        with contextlib.redirect_stderr(output):
            yield output


class CommandTest(unittest.TestCase):
    """Helpers for testing commands."""

    def setUp(self):
        super().setUp()

        # Make a temporary directory to work in, and rm-rf it when we're done.
        tempdir = tempfile.mkdtemp(suffix="_edx_lint_test")
        self.addCleanup(shutil.rmtree, tempdir)

        # Change into the temp directory, and change back when we are done.
        thisdir = os.getcwd()
        self.addCleanup(os.chdir, thisdir)
        os.chdir(tempdir)

        # Assure the file does not already exist.
        self.assert_not_file(PYLINTRC)

    def call_command(self, argv=None):
        """Call an edx_lint script command.

        Arguments:
            argv (list) -- arguments to pass to the edx_lint script
        """
        # This pointless statement is here because removing the statement causes a different warning. Currently we run
        # pylint at multiple versions in the Open edX platform.  However, the other warning (no-self-use) exists in the
        # old version of pylint (pre 2.13.0) but not the new version. This is the only way to fix the warning in a way
        # that will work for both the old and new version.
        self    # pylint: disable=pointless-statement
        return main.main(argv)

    def assert_file(self, filename, contains=None, not_contains=None):
        """Assert that a file exists, and optionally, contains some text."""
        self.assertTrue(os.path.isfile(filename))
        if contains is not None or not_contains is not None:
            with open(filename) as f:
                text = f.read()
                if contains is not None:  # pragma: no branch
                    self.assertIn(contains, text)
                if not_contains is not None:  # pragma: no branch
                    self.assertNotIn(not_contains, text)

    def assert_not_file(self, filename):
        """Assert that a file doesn't exist."""
        self.assertFalse(os.path.isfile(filename))


class WriteCommandTest(CommandTest):
    """Tests for the write command."""

    def test_write_arg_errors(self):
        self.assertEqual(1, self.call_command(["write"]))
        self.assertEqual(2, self.call_command(["write", "xyzzy"]))

    def test_write_creates_pylintrc(self):
        """ Verify the command writes a pylintrc file. """
        # Create the pylintrc.
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        self.assert_file(PYLINTRC, contains="*****\n# ** DO NOT EDIT THIS FILE **\n# *****")
        self.assert_file(PYLINTRC, not_contains="you must edit the central file", contains="LOCAL CHANGE")
        self.assert_file(PYLINTRC, contains="\n[MASTER]\n")
        self.assert_file(PYLINTRC, not_contains="\n# \n")

        # Try to create it again, it will verify it.
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))

    def test_write_backups_modified_file(self):
        # Make the pylintrc.
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        self.assert_file(PYLINTRC)
        self.assert_not_file(PYLINTRC_BACKUP)

        # Modify it.
        with open(PYLINTRC, "a") as pylintrc:
            pylintrc.write("# modified!\n")

        # Try to write again.
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        self.assert_file(PYLINTRC_BACKUP, contains="# modified!")

        # And if we write again, the previous backup is deleted.
        with open(PYLINTRC, "a") as pylintrc:
            pylintrc.write("# changed!\n")
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        self.assert_file(
            PYLINTRC_BACKUP, contains="# changed!", not_contains="# modified!"
        )

    def test_write_applies_tweaks(self):
        # Create a tweaks file.
        with open(PYLINTRC_TWEAKS, "w") as tweaks:
            tweaks.write("[MASTER]\nxyzzy = plugh\n")

        # Write the pylintrc.
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))

        # The tweak is in there!
        self.assert_file(PYLINTRC, contains="xyzzy = plugh")

    def test_file_types(self):
        self.assertEqual(0, self.call_command(["write", JUST_TESTING]))
        self.assert_file(JUST_TESTING, contains="\n-- ** DO NOT EDIT")
        self.assert_file(JUST_TESTING, contains="you must edit the central file", not_contains="LOCAL CHANGE")


class UpdateCommandTest(CommandTest):
    """Tests for the update command."""

    def test_no_files_exist(self):
        self.assertEqual(0, self.call_command(["update"]))
        self.assert_not_file(PYLINTRC)
        self.assert_not_file(EDITORCONFIG)

    def test_one_file_exists(self):
        # Write the initial version of pylintrc.
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        self.assert_file(PYLINTRC, contains="[MASTER]")
        self.assert_not_file(EDITORCONFIG)

        # Pretend edx-lint has a newer version of pylintrc, and update files.
        with patch("edx_lint.write.get_file_content") as get_file_content:
            get_file_content.return_value = "[xyz]\nsetting = True"
            self.assertEqual(0, self.call_command(["update"]))

        # pylintrc is re-written with new content.
        self.assert_file(PYLINTRC, contains="setting = True")
        self.assert_file(PYLINTRC, not_contains="[MASTER]")
        # .editorconfig still hasn't been written.
        self.assert_not_file(EDITORCONFIG)

    def test_update_backs_up_handwritten_files(self):
        # Create a hand-written pylintrc file:
        with open(PYLINTRC, "w") as pylintrc:
            pylintrc.write("[bogus]\nnot_really = pylintrc\n")

        # Update will see it's not one made by edx-lint, and move it aside.
        self.assertEqual(0, self.call_command(["update"]))

        # The backup file is our original hand-written file.
        self.assert_file(PYLINTRC_BACKUP, contains="not_really = pylintrc")
        # The pylintrc file was written by edx-lint.
        self.assert_file(PYLINTRC, contains="[MASTER]")


class CheckCommandTest(CommandTest):
    """Tests for the `check` command."""

    def test_happy_path(self):
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        with capture_output() as output:
            self.assertEqual(0, self.call_command(["check", PYLINTRC]))
        expected = "pylintrc is good\n"
        self.assertEqual(output.getvalue(), expected)

    def test_handwritten_is_ok(self):
        # Create a hand-written pylintrc file:
        with open(PYLINTRC, "w") as pylintrc:
            pylintrc.write("[bogus]\nnot_really = pylintrc\n")

        with capture_output() as output:
            self.assertEqual(0, self.call_command(["check", PYLINTRC]))
        expected = "pylintrc wasn't written by edx_lint\n"
        self.assertEqual(output.getvalue(), expected)

    def test_handedited_fails(self):
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        with open(PYLINTRC, "a") as pylintrc:
            pylintrc.write("More stuff by hand\n")

        with capture_output() as output:
            self.assertEqual(1, self.call_command(["check", PYLINTRC]))
        expected = "pylintrc seems to have been edited\n"
        self.assertEqual(output.getvalue(), expected)

    def test_all_files(self):
        self.assertEqual(0, self.call_command(["write", PYLINTRC]))
        self.assertEqual(0, self.call_command(["write", EDITORCONFIG]))

        with capture_output() as output:
            self.assertEqual(0, self.call_command(["check"]))
        expected = (
            "pylintrc is good\n" +
            ".editorconfig is good\n"
        )
        self.assertEqual(output.getvalue(), expected)

    def test_cant_check_a_missing_file(self):
        with capture_output() as output:
            self.assertEqual(2, self.call_command(["check", "xyzzy.foo"]))
        self.assertEqual(output.getvalue(), "xyzzy.foo doesn't exist\n")



"""Checker for using pytest assertion instead of unittest assertion."""
import astroid

from pylint.interfaces import IAstroidChecker
from pylint.checkers import BaseChecker, utils

from edx_lint.pylint.common import BASE_ID, check_visitors


def register_checkers(linter):
    """Register checkers."""
    linter.register_checker(UnittestAssertChecker(linter))


@check_visitors
class UnittestAssertChecker(BaseChecker):
    """
    Checks if a unit test assertion is used, Trigger warning to
    replace it with pytest assertions
    """

    __implements__ = (IAstroidChecker,)

    name = "unittest-assert-checker"

    UNITTEST_ASSERTS = [
        "assertTrue",
        "assertFalse",
        "assertEqual",
        "assertEquals",
        "assertNotEqual",
        "assertNotEquals",
        "assert_",
        "assertIn",
        "assertNotIn",
        "assertLess",
        "assertLessEqual",
        "assertGreater",
        "assertGreaterEqual",
        "assertAlmostEqual",
        "assertNotAlmostEqual",
        "assertIs",
        "assertIsNot",
        "assertIsNone",
        "assertIsNotNone",
        "assertIsInstance",
        "assertNotIsInstance",
        "assertRaises",
    ]

    ASSERT_MAPPING = {
        "assertEqual": "assert arg1 == arg2",
        "assertEquals": "assert arg1 == arg2",
        "assertNotEqual": "assert arg1 != arg2",
        "assertNotEquals": "assert arg1 != arg2",
        "assert_": "assert arg1",
        "assertTrue": "assert arg1",
        "assertFalse": "assert not arg1",
        "assertIn": "assert arg1 in arg2",
        "assertNotIn": "assert arg1 not in arg2",
        "assertIs": "assert arg1 is arg2",
        "assertIsNot": "assert arg1 is not arg2",
        "assertIsNone": "assert arg1 is None",
        "assertIsNotNone": "assert arg1 is not None",
        "assertIsInstance": "assert isinstance(arg1, arg2)",
        "assertNotIsInstance": "assert not isinstance(arg1, arg2)",
        "assertLess": "assert arg1 < arg2",
        "assertLessEqual": "assert arg1 <= arg2",
        "assertGreater": "assert arg1 > arg2",
        "assertGreaterEqual": "assert arg1 >= arg2",
        "assertAlmostEqual": "assert math.isclose(arg1, arg2)",
        "assertNotAlmostEqual": "assert not math.isclose(arg1, arg2)",
        "assertRaises": "pytest.raises(arg) or with pytest.raises(arg) as optional_var:",
    }

    MESSAGE_ID = "avoid-unittest-asserts"
    msgs = {
        ("C%d99" % BASE_ID): (
            "%s",
            MESSAGE_ID,
            "Avoid using unittest's assertion methods when using pytest, instead use the 'assert' statement"
        )
    }

    @utils.check_messages(MESSAGE_ID)
    def visit_call(self, node):
        """
        Check that unittest assertions are not used.
        """
        if not isinstance(node.func, astroid.Attribute):
            # If it isn't a getattr ignore this. All the assertMethods are
            # attributes of self:
            return

        if node.func.attrname not in self.UNITTEST_ASSERTS:
            # Not an attribute we care about
            return

        converted_assert = self.ASSERT_MAPPING.get(node.func.attrname, None)

        self.add_message(
            self.MESSAGE_ID,
            args=f"{node.func.attrname} should be replaced with a pytest assertion something like `{converted_assert}`",
            node=node
        )



"""Config file tools for edx_lint."""


def merge_configs(main, tweaks):
    """Merge tweaks into a main config file."""
    for section in tweaks.sections():
        for option in tweaks.options(section):
            value = tweaks.get(section, option)
            if option.endswith("+"):
                option = option[:-1]
                value = main.get(section, option) + value
            main.set(section, option, value)



"""Test super_check.py"""

import pytest

from .pylint_test import run_pylint

MSG_IDS = "super-method-not-called,non-parent-method-called"


@pytest.mark.parametrize("method", ["setUp", "tearDown", "setUpClass", "tearDownClass"])
def test_unittest_super_check(method):
    source = (
        """\
        import unittest

        class GoodTestCase(unittest.TestCase):
            def {method}(self):
                super().{method}()

        class BadTestCase(unittest.TestCase):
            def {method}(self):         #=A
                self.i_am_bad = True

        class OldSchool(Mixin, unittest.TestCase):
            def {method}(self):
                Mixin.{method}(self)
                unittest.TestCase.{method}(self)

        def {method}(xyzzy):
            # Weird, but who cares?
            pass
        """
    ).format(method=method)
    messages = run_pylint(source, MSG_IDS)
    expected = {
        "14:non-parent-method-called:{}() was called from a non-parent class (unittest.case.TestCase)".format(
            method
        )
    }
    assert expected == messages


@pytest.mark.parametrize("method", ["setUpTestData"])
def test_django_super_check(method):
    source = (
        """\
        import django

        class GoodTestCase(django.test.TestCase):
            def {method}(self):
                super().{method}()

        class BadTestCase(django.test.TestCase):
            def {method}(self):         #=A
                self.i_am_bad = True
        """
    ).format(method=method)
    messages = run_pylint(source, MSG_IDS)
    expected = set()
    assert expected == messages


def test_hamfisted_super():
    source = """\
        import unittest

        class SomeOtherClass(object):
            def setUp(self):
                pass

        class BadTestCase(unittest.TestCase):
            def setUp(self):
                foo("What").setUp(self)
                What.setUp(self)
                SomeOtherClass.setUp(self)   #=A
                super().setUp()
        """
    messages = run_pylint(source, MSG_IDS)
    expected = {
        "A:non-parent-method-called:setUp() was called from a non-parent class (source.SomeOtherClass)"
    }
    assert expected == messages


def test_good_super():
    source = """\
        import unittest

        def foo_func(bar):
            pass

        class GoodTestCase(unittest.TestCase):
            def setUp(self):
                self.foo_meth(bar)
                foo_func(bar)
                super().setUp()

            def tearDown(self):
                # Not sure why you would do it this way, ...
                base = super()
                base.tearDown()

            def foo_meth(self, bar):
                foo_func(bar)

            def test_something(self):
                pass

        class NotSureWhatThisIs(object):
            def setUp(self):
                pass
        """
    messages = run_pylint(source, MSG_IDS)
    assert not messages



"""
Commandline scripts to give one-time amnesty for specify linting error conditions.
"""



import logging
import re
import sys
from collections import namedtuple, defaultdict

import click
import click_log

LOG = logging.getLogger(__name__)
PYLINT_PARSEABLE_REGEX = re.compile(
    r"""^(?P<filename>[^:]+):(?P<linenum>\d+): """
    r"""\[(?P<error_code>[^(]+)\((?P<error_name>[^)]+)\), (?P<function>[^\]]*)\] """
    r"""(?P<error_msg>.*)"""
)
PYLINT_EXCEPTION_REGEX = re.compile(r"""\s*#\s*pylint:\s+disable=(?P<disables>[^#$]+?)(?=\s*(#|$))""")

PylintError = namedtuple("PylintError", ["filename", "linenum", "error_code", "error_name", "function", "error_msg"])


def parse_pylint_output(pylint_output):
    """
    Parse the pylint output-format=parseable lines into PylintError tuples.
    """
    for line in pylint_output:
        if not line.strip():
            continue

        if line[0:5] in ("-" * 5, "*" * 5):
            continue

        parsed = PYLINT_PARSEABLE_REGEX.search(line)
        if parsed is None:
            LOG.warning(
                "Unable to parse %r. If this is a lint failure, please re-run pylint with the "
                "--output-format=parseable option, otherwise, you can ignore this message.",
                line,
            )
            continue

        parsed_dict = parsed.groupdict()
        parsed_dict["linenum"] = int(parsed_dict["linenum"])
        yield PylintError(**parsed_dict)


def format_pylint_disables(error_names, tag=True):
    """
    Format a list of error_names into a 'pylint: disable=' line.
    """
    tag_str = "lint-amnesty, " if tag else ""
    if error_names:
        return "  # {tag}pylint: disable={disabled}".format(disabled=", ".join(sorted(error_names)), tag=tag_str)
    else:
        return ""


def fix_pylint(line, errors):
    """
    Yield any modified versions of ``line`` needed to address the errors in ``errors``.
    """
    if not errors:
        yield line
        return

    current = PYLINT_EXCEPTION_REGEX.search(line)
    if current:
        original_errors = {disable.strip() for disable in current.group("disables").split(",")}
    else:
        original_errors = set()

    disabled_errors = set(original_errors)

    for error in errors:
        if error.error_name == "useless-suppression":
            parsed = re.search("""Useless suppression of '(?P<error_name>[^']+)'""", error.error_msg)
            disabled_errors.discard(parsed.group("error_name"))
        elif error.error_name == "missing-docstring" and error.error_msg == "Missing module docstring":
            yield format_pylint_disables({error.error_name}).strip() + "\n"
        else:
            disabled_errors.add(error.error_name)

    disable_string = format_pylint_disables(disabled_errors, not disabled_errors <= original_errors)

    if current:
        yield PYLINT_EXCEPTION_REGEX.sub(disable_string, line)
    else:
        yield re.sub(r"($\s*)", disable_string + r"\1", line, count=1)


@click.command()
@click.option(
    "--pylint-output",
    default=sys.stdin,
    type=click.File(),
    help="An input file containing pylint --output-format=parseable errors. Defaults to stdin.",
)
@click_log.simple_verbosity_option(default="INFO")
def pylint_amnesty(pylint_output):
    """
    Add ``# pylint: disable`` clauses to add exceptions to all existing pylint errors in a codebase.
    """
    errors = defaultdict(lambda: defaultdict(set))
    for pylint_error in parse_pylint_output(pylint_output):
        errors[pylint_error.filename][pylint_error.linenum].add(pylint_error)

    for file_with_errors in sorted(errors):
        try:
            opened_file = open(file_with_errors)
        except OSError:
            LOG.warning("Unable to open %s for edits", file_with_errors, exc_info=True)
        else:
            with opened_file as input_file:
                output_lines = []
                for line_num, line in enumerate(input_file, start=1):
                    # If the line ends with backslash, take the amnesty to next line because line of code has not ended
                    if line.endswith('\\\n'):
                        errors[file_with_errors][line_num + 1] = errors[file_with_errors][line_num]
                        errors[file_with_errors][line_num] = set()

                    output_lines.extend(fix_pylint(line, errors[file_with_errors][line_num]))

            with open(file_with_errors, "w") as output_file:
                output_file.writelines(output_lines)
